@@ -26,6 +26,7 @@ DEALINGS IN THE SOFTWARE.
 from __future__ import annotations
 
 import asyncio
+import contextlib
 import enum
 import hashlib
 import json
@@ -1416,8 +1417,7 @@ class Socket:
         self.activity_timeout: int = 120
         self.last_message: float = 0
         self.last_ping: float = 0
-        self.ponged: bool = True
-        self.pinged: bool = False
+        self.last_pong: float = 1
         self.subscriptions: Set[Subscription[Any]] = set()
         self.channels: Dict[str, Subscription[Any]] = {}
         self.close_code: Optional[int] = None
@@ -1442,6 +1442,10 @@ class Socket:
             return
         else:
             self.reconnecting = True
+        # ensure the ping, pong loop doesn't close the connection due to thinking it timed out when the connection was broken and needs to be reconnected
+        self.last_message = time.perf_counter()
+        self.last_ping = time.perf_counter()
+        self.last_pong = time.perf_counter() + 1
         try:
             if self.kit.aiohttp_session is None:
                 self.kit.aiohttp_session = aiohttp.ClientSession()
@@ -1452,7 +1456,6 @@ class Socket:
                 autoclose=False,
                 timeout=30,
             )
-            self.ponged = True
             for subscription in self.subscriptions:
                 subscription.succeeded.clear()
                 await self.subscribe(subscription)
@@ -1487,8 +1490,7 @@ class Socket:
                                 continue
                             subscription.succeeded.set()
                         elif event == "pusher:pong":
-                            self.ponged = True
-                            self.pinged = False
+                            self.last_pong = time.perf_counter()
                         elif event == "pusher:ping":
                             await self.ws.send_json(
                                 {"event": "pusher:pong", "data": {}}
@@ -1500,6 +1502,12 @@ class Socket:
                             if subscription is None:
                                 continue
                             subscription.handle_event(event, data)
+                    except ConnectionResetError as e:
+                        utils.print_exception_with_header(
+                            "Encountered ConnectionResetError in socket", e
+                        )
+                        await self.close_and_reconnect()
+                        break
                     except Exception as e:
                         utils.print_exception_with_header(
                             "Ignoring exception when parsing WebSocket message", e
@@ -1513,39 +1521,49 @@ class Socket:
     async def handle_socket_close(self) -> None:
         close_code = self.ws.close_code or self.close_code
         if close_code is None or close_code in range(4000, 4100):
-            raise errors.NoReconnect(
-                f"WebSocket closed with close code {close_code}"
-            )
+            raise errors.NoReconnect(f"WebSocket closed with close code {close_code}")
         elif close_code in range(4100, 4200):
             await asyncio.sleep(1)
             await self.reconnect()
         else:
             await self.reconnect()
 
-    async def async_call_later_pong(self) -> None:
+    async def close_and_reconnect(self, message: bytes = b"Pong timeout") -> None:
         self.close_code = 1002
-        await self.ws.close(code=1002, message=b"Pong timeout")
+        with contextlib.suppress(ConnectionResetError):
+            await self.ws.close(code=1002, message=message)
         await self.reconnect()
 
-    def call_later_pong(self) -> None:
-        if not self.ponged:
-            asyncio.create_task(self.async_call_later_pong())
-
     async def ping_pong(self) -> None:
         while True:
-            await asyncio.sleep(
-                self.last_message + self.activity_timeout - time.perf_counter()
-            )
-            if (
-                self.last_message + self.activity_timeout > time.perf_counter()
-                or self.pinged
-            ):
-                continue
-            await self.ws.send_json({"event": "pusher:ping", "data": {}})
-            self.ponged = False
-            self.pinged = True
-            self.last_ping = time.perf_counter()
-            asyncio.get_running_loop().call_later(30, self.call_later_pong)
+            try:
+                # sleeps until next ping
+                await asyncio.sleep(
+                    self.last_message + self.activity_timeout - time.perf_counter()
+                )
+                # if received a message within activity timeout, continue to next iteration since we do not need to ping
+                if self.last_message + self.activity_timeout > time.perf_counter():
+                    continue
+                if self.last_ping > self.last_pong:
+                    # if pinged, not received pong, passed timeout, close and reconnect
+                    if self.last_ping + self.activity_timeout > time.perf_counter():
+                        # has pinged, has not received pong, and has not received any other messages in activity_timeout
+                        # therefore, connection is dead and should be closed and reconnected
+                        await self.close_and_reconnect()
+                        await asyncio.sleep(self.activity_timeout)
+                    # otherwise just continue to next iteration
+                    continue
+                # don't send ping if last ping was within activity_timeout
+                if self.last_ping + self.activity_timeout > time.perf_counter():
+                    continue
+                # don't send ping if last ping was not ponged
+                if self.last_pong < self.last_ping:
+                    continue
+                await self.ws.send_json({"event": "pusher:ping", "data": {}})
+                self.last_ping = time.perf_counter()
+
+            except ConnectionResetError:
+                asyncio.create_task(self.close_and_reconnect(b"Connection reset"))
 
     def run(self) -> None:
         self.task = asyncio.create_task(self.actual_run())
@@ -1570,7 +1588,13 @@ class Socket:
         )
 
     async def send(self, event: str, data: Dict[str, Any]) -> None:
-        await self.ws.send_json({"event": event, "data": data})
+        if self.ws.closed:
+            await self.close_and_reconnect()
+        try:
+            await self.ws.send_json({"event": event, "data": data})
+        except ConnectionResetError:
+            await self.close_and_reconnect()
+            await self.ws.send_json({"event": event, "data": data})
 
     async def subscribe(self, subscription: Subscription[Any]) -> None:
         try:
